@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,16 +18,14 @@
 #define _ND_NDPI_H
 
 #include <ndpi_api.h>
+#include <ndpi_main.h>
+#include <ndpi_typedefs.h>
 
 void ndpi_global_init(void);
-void ndpi_global_destroy(void);
 
-struct ndpi_detection_module_struct *nd_ndpi_init(
-    const string &tag, uint32_t &custom_proto_base);
+struct ndpi_detection_module_struct *nd_ndpi_init(void);
 
 void nd_ndpi_free(struct ndpi_detection_module_struct *ndpi);
 
-struct ndpi_detection_module_struct *ndpi_get_parent(void);
-
 #endif // _ND_NDPI_H
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
