@@ -17,8 +17,8 @@ static void ndpi_int_rtcp_add_connection(struct ndpi_detection_module_struct
 			     NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 }
 
-void ndpi_search_rtcp(struct ndpi_detection_module_struct *ndpi_struct,
-		      struct ndpi_flow_struct *flow)
+static void ndpi_search_rtcp(struct ndpi_detection_module_struct *ndpi_struct,
+			     struct ndpi_flow_struct *flow)
 {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   u_int16_t dport = 0, sport = 0;
@@ -45,9 +45,10 @@ void ndpi_search_rtcp(struct ndpi_detection_module_struct *ndpi_struct,
       len = packet->payload[2+offset] * 256 + packet->payload[2+offset+1];
       rtcp_section_len = (len + 1) * 4;
       
-      if(((offset+rtcp_section_len) > packet->payload_packet_len) || (rtcp_section_len == 0) || (len == 0))
-	goto exclude_rtcp;
-      else
+      if(((offset+rtcp_section_len) > packet->payload_packet_len) || (rtcp_section_len == 0) || (len == 0)) {
+        NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
+	return;
+      } else
 	offset += rtcp_section_len;
     }
     
@@ -63,17 +64,13 @@ void ndpi_search_rtcp(struct ndpi_detection_module_struct *ndpi_struct,
 
     if(flow->packet_counter > 3)
       NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
-  } else {
-  exclude_rtcp:
-    
-    NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
   }
 }
 
 
-void init_rtcp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask)
+void init_rtcp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id)
 {
-  ndpi_set_bitmask_protocol_detection("RTCP", ndpi_struct, detection_bitmask, *id,
+  ndpi_set_bitmask_protocol_detection("RTCP", ndpi_struct, *id,
 				      NDPI_PROTOCOL_RTCP,
 				      ndpi_search_rtcp,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
