@@ -36,8 +36,8 @@ static void ndpi_int_tftp_add_connection(struct ndpi_detection_module_struct
   ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_TFTP, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 }
 
-void ndpi_search_tftp(struct ndpi_detection_module_struct
-		      *ndpi_struct, struct ndpi_flow_struct *flow)
+static void ndpi_search_tftp(struct ndpi_detection_module_struct *ndpi_struct,
+			     struct ndpi_flow_struct *flow)
 {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
@@ -156,9 +156,9 @@ void ndpi_search_tftp(struct ndpi_detection_module_struct
 }
 
 
-void init_tftp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask)
+void init_tftp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id)
 {
-  ndpi_set_bitmask_protocol_detection("TFTP", ndpi_struct, detection_bitmask, *id,
+  ndpi_set_bitmask_protocol_detection("TFTP", ndpi_struct, *id,
 				      NDPI_PROTOCOL_TFTP,
 				      ndpi_search_tftp,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_UDP_WITH_PAYLOAD,
