@@ -53,6 +53,10 @@
 #define __FAVOR_BSD 1
 #include <netinet/ip.h>
 
+#include <net/if.h>
+#include <net/if_arp.h>
+#include <linux/if_packet.h>
+
 #include <pcap/pcap.h>
 
 #include <curl/curl.h>
@@ -65,17 +69,20 @@ using json = nlohmann::json;
 using namespace std;
 
 #include "netifyd.h"
-#ifdef _ND_USE_NETLINK
-#include "nd-netlink.h"
-#endif
 #include "nd-ndpi.h"
+#include "nd-risks.h"
+#include "nd-serializer.h"
+#include "nd-packet.h"
 #include "nd-json.h"
 #include "nd-thread.h"
 #include "nd-util.h"
+#include "nd-addr.h"
+#ifdef _ND_USE_NETLINK
+#include "nd-netlink.h"
+#endif
 #include "nd-apps.h"
 #include "nd-category.h"
 #include "nd-protos.h"
-#include "nd-risks.h"
 #include "nd-flow.h"
 #include "nd-flow-map.h"
 #include "nd-flow-parser.h"
@@ -85,145 +92,8 @@ using namespace std;
 
 ndApplications *nd_apps = NULL;
 
-template<size_t N>
-bool operator<(const bitset<N> &x, const bitset<N> &y)
-{
-    for (int i = N-1; i >= 0; i--) {
-        if (x[i] ^ y[i]) return y[i];
-    }
-
-    return false;
-}
-
-template<size_t N>
-bitset<N> &operator-=(bitset<N> &x, const size_t y)
-{
-    bool borrow = false;
-    bitset<N> const _y(y);
-
-    for (size_t i = 0; i < N; i++) {
-        if (borrow) {
-            if (x[i]) {
-                x[i] = _y[i];
-                borrow = _y[i];
-            }
-            else {
-                x[i] = ! _y[i];
-                borrow = true;
-            }
-        }
-        else {
-            if (x[i]) {
-                x[i] = ! _y[i];
-                borrow = false;
-            }
-            else {
-                x[i] = _y[i];
-                borrow = _y[i];
-            }
-        }
-    }
-
-    return x;
-}
-
-template <size_t N>
-class ndRadixNetworkEntry {
-public:
-    bitset<N> addr;
-    size_t prefix_len;
-
-    ndRadixNetworkEntry() : prefix_len(0) { }
-
-    bool operator[] (int n) const {
-        return addr[(N - 1) - n];
-    }
-
-    bool operator== (const ndRadixNetworkEntry &rhs) const {
-        return prefix_len == rhs.prefix_len && addr == rhs.addr;
-    }
-
-    bool operator< (const ndRadixNetworkEntry &rhs) const {
-        if (addr == rhs.addr)
-            return prefix_len < rhs.prefix_len;
-        else
-            return addr < rhs.addr;
-    }
-#ifndef _ND_LEAN_AND_MEAN
-    bool to_ipstring(sa_family_t af, string &ip) const {
-        struct in_addr nw_addr;
-        struct in6_addr nw6_addr;
-        switch (af) {
-        case AF_INET:
-            nw_addr.s_addr = htonl(addr.to_ulong());
-            if (! nd_ip_to_string(
-                AF_INET, (const void *)&nw_addr, ip))
-                return false;
-            break;
-        case AF_INET6:
-            for (auto i = 0; i < 4; i++) {
-                bitset<32> b;
-                for (auto j = 0; j < 32; j++)
-                    b[j] = addr[i * 32 + j];
-                nw6_addr.s6_addr32[3 - i] = htonl(b.to_ulong());
-            }
-            if (! nd_ip_to_string(
-                AF_INET6, (const void *)&nw6_addr, ip))
-                return false;
-            break;
-        default:
-            return false;
-        }
-
-        ip.append(string("/") + to_string(prefix_len));
-        return true;
-    }
-#endif
-};
-
-template <size_t N>
-static ndRadixNetworkEntry<N> radix_substr(
-    const ndRadixNetworkEntry<N> &entry, int offset, int length
-) {
-    bitset<N> mask;
-
-    if (length == N)
-        mask = 0;
-    else
-        mask = 1 << length;
-
-    mask -= 1;
-    mask <<= N - length - offset;
-
-    ndRadixNetworkEntry<N> result;
-    result.addr = (entry.addr & mask) << offset;
-    result.prefix_len = length;
-
-    return result;
-}
-
-template <size_t N>
-static ndRadixNetworkEntry<N> radix_join(
-    const ndRadixNetworkEntry<N> &x,
-    const ndRadixNetworkEntry<N> &y
-) {
-    ndRadixNetworkEntry<N> result;
-
-    result.addr = x.addr;
-    result.addr |= y.addr >> x.prefix_len;
-    result.prefix_len = x.prefix_len + y.prefix_len;
-
-    return result;
-}
-
-template <size_t N>
-static int radix_length(const ndRadixNetworkEntry<N> &entry)
-{
-    return (int)entry.prefix_len;
-}
-
-typedef radix_tree<ndRadixNetworkEntry<32>, nd_app_id_t> nd_rn4_t;
-typedef radix_tree<ndRadixNetworkEntry<128>, nd_app_id_t> nd_rn6_t;
+typedef radix_tree<ndRadixNetworkEntry<32>, nd_app_id_t> nd_rn4_app;
+typedef radix_tree<ndRadixNetworkEntry<128>, nd_app_id_t> nd_rn6_app;
 
 ndApplications::ndApplications()
     : app_networks4(NULL), app_networks6(NULL)
@@ -404,18 +274,18 @@ bool ndApplications::Save(const string &filename)
         ofs << "app:" << it.first << ":" << it.second->tag << endl;
     for (auto &it : domains)
         ofs << "dom:" << it.second << ":" << it.first << endl;
-    nd_rn4_t *rn4 = static_cast<nd_rn4_t *>(app_networks4);
+    nd_rn4_app *rn4 = static_cast<nd_rn4_app *>(app_networks4);
     for (auto &it : (*rn4)) {
         string ip;
-        if (it.first.to_ipstring(AF_INET, ip)) {
+        if (it.first.GetString(ip)) {
             ofs << "net:" << it.second << ":" << ip << endl;
             nc++;
         }
     }
-    nd_rn6_t *rn6 = static_cast<nd_rn6_t *>(app_networks6);
+    nd_rn6_app *rn6 = static_cast<nd_rn6_app *>(app_networks6);
     for (auto &it : (*rn6)) {
         string ip;
-        if (it.first.to_ipstring(AF_INET6, ip)) {
+        if (it.first.GetString(ip)) {
             ofs << "net:" << it.second << ":" << ip << endl;
             nc++;
         }
@@ -429,7 +299,7 @@ bool ndApplications::Save(const string &filename)
 
     return true;
 #else
-    nd_printf("Sorry, this feature was disabled (lean and mean).\n");
+    nd_printf("Sorry, this feature was disabled (embedded).\n");
     return false;
 #endif // _ND_LEAN_AND_MEAN
 }
@@ -472,47 +342,35 @@ nd_app_id_t ndApplications::Find(const string &domain)
     return ND_APP_UNKNOWN;
 }
 
-nd_app_id_t ndApplications::Find(sa_family_t af, void *addr)
+nd_app_id_t ndApplications::Find(const ndAddr &addr)
 {
-    unique_lock<mutex> ul(lock);
-
-    struct in_addr *dst_addr;
-    struct in6_addr *dst6_addr;
-
-    switch (af) {
-    case AF_INET:
-        dst_addr = static_cast<struct in_addr *>(addr);
-        break;
-    case AF_INET6:
-        dst6_addr = static_cast<struct in6_addr *>(addr);
-        break;
-    default:
-        nd_printf("Invalid address family: %hu\n", af);
+    if (! addr.IsValid() || ! addr.IsIP())
         return ND_APP_UNKNOWN;
-    }
 
-    if (af == AF_INET) {
+    if (addr.IsIPv4()) {
         ndRadixNetworkEntry<32> entry;
-        entry.prefix_len = 32;
-        entry.addr = ntohl(dst_addr->s_addr);
+        if (ndRadixNetworkEntry<32>::CreateQuery(entry, addr)) {
 
-        nd_rn4_t::iterator it;
-        nd_rn4_t *rn4 = static_cast<nd_rn4_t *>(app_networks4);
-        if ((it = rn4->longest_match(entry)) != rn4->end())
-            return it->second;
-    }
-    else if (af == AF_INET6) {
-        ndRadixNetworkEntry<128> entry;
-        entry.prefix_len = 128;
-        for (auto i = 0; i < 4; i++) {
-            entry.addr |= ntohl(dst6_addr->s6_addr32[i]);
-            if (i != 3) entry.addr <<= 32;
+            unique_lock<mutex> ul(lock);
+
+            nd_rn4_app::iterator it;
+            nd_rn4_app *rn4 = static_cast<nd_rn4_app *>(app_networks4);
+            if ((it = rn4->longest_match(entry)) != rn4->end())
+                return it->second;
         }
+    }
 
-        nd_rn6_t::iterator it;
-        nd_rn6_t *rn6 = static_cast<nd_rn6_t *>(app_networks6);
-        if ((it = rn6->longest_match(entry)) != rn6->end())
-            return it->second;
+    if (addr.IsIPv6()) {
+        ndRadixNetworkEntry<128> entry;
+        if (ndRadixNetworkEntry<128>::CreateQuery(entry, addr)) {
+
+            unique_lock<mutex> ul(lock);
+
+            nd_rn6_app::iterator it;
+            nd_rn6_app *rn6 = static_cast<nd_rn6_app *>(app_networks6);
+            if ((it = rn6->longest_match(entry)) != rn6->end())
+                return it->second;
+        }
     }
 
     return ND_APP_UNKNOWN;
@@ -564,20 +422,20 @@ bool ndApplications::Lookup(nd_app_id_t id, ndApplication &app)
 void ndApplications::Reset(bool free_only)
 {
     if (app_networks4 != nullptr) {
-        nd_rn4_t *rn4 = static_cast<nd_rn4_t *>(app_networks4);
+        nd_rn4_app *rn4 = static_cast<nd_rn4_app *>(app_networks4);
         delete rn4;
         app_networks4 = NULL;
     }
 
     if (app_networks6 != nullptr) {
-        nd_rn6_t *rn6 = static_cast<nd_rn6_t *>(app_networks6);
+        nd_rn6_app *rn6 = static_cast<nd_rn6_app *>(app_networks6);
         delete rn6;
         app_networks6 = NULL;
     }
 
     if (! free_only) {
-        nd_rn4_t *rn4 = new nd_rn4_t;
-        nd_rn6_t *rn6 = new nd_rn6_t;
+        nd_rn4_app *rn4 = new nd_rn4_app;
+        nd_rn6_app *rn6 = new nd_rn6_app;
 
         if (rn4 == nullptr || rn6 == nullptr) {
             throw ndSystemException(
@@ -636,7 +494,8 @@ ndApplication *ndApplications::AddApp(
 bool ndApplications::AddDomain(nd_app_id_t id, const string &domain)
 {
     auto rc = domains.insert(make_pair(domain, id));
-    if (domain.find_first_of(".") == string::npos) tlds.insert(domain);
+    if (domain.find_first_of(".") == string::npos)
+        tlds.insert(domain);
     return rc.second;
 }
 
@@ -669,80 +528,34 @@ bool ndApplications::AddDomainTransform(const string &search, const string &repl
 bool ndApplications::AddNetwork(
     nd_app_id_t id, const string &network
 ) {
-    struct in_addr nw_addr;
-    struct in6_addr nw6_addr;
-    sa_family_t af = AF_UNSPEC;
-    size_t shift, prefix_max = 0, prefix_len = 0;
+    ndAddr addr(network);
 
-    string addr;
-    size_t p = string::npos;
-    if ((p = network.find_first_of("/")) != string::npos) {
-        addr = network.substr(0, p);
-        prefix_len = (size_t)strtoul(
-            network.substr(p + 1).c_str(), NULL, 0
-        );
-    }
-
-    if (inet_pton(AF_INET, addr.c_str(), &nw_addr)) {
-        af = AF_INET;
-        prefix_max = 32;
-    }
-    else if (inet_pton(AF_INET6, addr.c_str(), &nw6_addr)) {
-        af = AF_INET6;
-        prefix_max = 128;
-    }
-    else {
-        nd_printf("Invalid IPv4/6 network address: %s\n", addr.c_str());
+    if (! addr.IsValid() || ! addr.IsIP()) {
+        nd_printf("Invalid IPv4/6 network address: %s\n", network.c_str());
         return false;
     }
 
-    if (prefix_len > prefix_max) {
-        nd_printf("Invalid prefix length: > %u\n", prefix_max);
-        return false;
-    }
-
-    bitset<32> mask32;
-    bitset<128> mask128;
-
-    shift = prefix_max - prefix_len;
-    if (shift < prefix_max) {
-        if (prefix_max == 32) {
-            mask32.set();
-            for (size_t i = 0; i < shift; i++) mask32.flip(i);
-        }
-        else {
-            mask128.set();
-            for (size_t i = 0; i < shift; i++) mask128.flip(i);
-        }
-    }
-
     try {
-        if (af == AF_INET) {
+        if (addr.addr.ss.ss_family == AF_INET) {
             ndRadixNetworkEntry<32> entry;
-            entry.prefix_len = prefix_len;
-            entry.addr = ntohl(nw_addr.s_addr);
-            entry.addr &= mask32;
-
-            nd_rn4_t *rn4 = static_cast<nd_rn4_t *>(app_networks4);
-            (*rn4)[entry] = id;
-            return true;
+            if (ndRadixNetworkEntry<32>::Create(entry, addr)) {
+                nd_rn4_app *rn4 = static_cast<nd_rn4_app *>(app_networks4);
+                (*rn4)[entry] = id;
+                return true;
+            }
         }
         else {
             ndRadixNetworkEntry<128> entry;
-            entry.prefix_len = prefix_len;
-            for (auto i = 0; i < 4; i++) {
-                entry.addr |= ntohl(nw6_addr.s6_addr32[i]);
-                if (i != 3) entry.addr <<= 32;
+            if (ndRadixNetworkEntry<128>::Create(entry, addr)) {
+                nd_rn6_app *rn6 = static_cast<nd_rn6_app *>(app_networks6);
+                (*rn6)[entry] = id;
+                return true;
             }
-            entry.addr &= mask128;
-
-            nd_rn6_t *rn6 = static_cast<nd_rn6_t *>(app_networks6);
-            (*rn6)[entry] = id;
-            return true;
         }
     }
     catch (runtime_error &e) {
-        nd_dprintf("Error adding network: %s: %s\n", network.c_str(), e.what());
+        nd_dprintf("Error adding network: %s: %s\n",
+            network.c_str(), e.what());
     }
 
     return false;
