@@ -29,6 +29,7 @@
 #include <atomic>
 #include <regex>
 #include <mutex>
+#include <bitset>
 
 #include <sys/stat.h>
 #include <sys/select.h>
@@ -42,7 +43,6 @@
 #include <string.h>
 #include <time.h>
 #include <arpa/inet.h>
-#include <linux/if_ether.h>
 #ifdef _ND_USE_NETLINK
 #include <linux/netlink.h>
 #endif
@@ -51,6 +51,11 @@
 #include <netinet/tcp.h>
 #undef __FAVOR_BSD
 
+#include <net/if.h>
+#include <net/if_arp.h>
+#include <linux/if_ether.h>
+#include <linux/if_packet.h>
+
 #include <pcap/pcap.h>
 
 #include <libmnl/libmnl.h>
@@ -59,28 +64,36 @@
 #include <nlohmann/json.hpp>
 using json = nlohmann::json;
 
+#include <radix/radix_tree.hpp>
+
 using namespace std;
 
 #include "netifyd.h"
 
+#include "nd-config.h"
 #include "nd-ndpi.h"
+#include "nd-risks.h"
+#include "nd-serializer.h"
+#include "nd-packet.h"
+#include "nd-json.h"
+#include "nd-util.h"
+#include "nd-addr.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
-#include "nd-json.h"
 #include "nd-apps.h"
 #include "nd-protos.h"
-#include "nd-risks.h"
 #include "nd-category.h"
 #include "nd-flow.h"
 #include "nd-thread.h"
-#include "nd-util.h"
 
 // Enable Conntrack debug logging
 //#define _ND_DEBUG_CONNTRACK     1
 
 #include "nd-conntrack.h"
 
+extern ndGlobalConfig nd_config;
+
 static time_t nd_ct_last_flow_purge_ttl = 0;
 
 static int nd_ct_event_callback(
@@ -522,10 +535,10 @@ void ndConntrackThread::PrintFlow(ndFlow *flow, string &text)
         family, flow->ip_protocol);
 
     os << buffer;
-    os << ", lower_ip: " << flow->lower_ip;
-    os << ", upper_ip: " << flow->upper_ip;
-    os << ", lower_port: " << ntohs(flow->lower_port);
-    os << ", upper_port: " << ntohs(flow->upper_port);
+    os << ", lower_ip: " << flow->lower_addr.GetString();
+    os << ", upper_ip: " << flow->upper_addr.GetString();
+    os << ", lower_port: " << flow->lower_addr.GetPort();
+    os << ", upper_port: " << flow->upper_addr.GetPort();
 
     text = os.str();
 }
@@ -552,25 +565,15 @@ void ndConntrackThread::UpdateFlow(ndFlow *flow)
     sha1_write(&ctx, (const char *)&family, sizeof(sa_family_t));
     sha1_write(&ctx, (const char *)&flow->ip_protocol, sizeof(uint8_t));
 
-    switch (family) {
-    case AF_INET:
-        sha1_write(&ctx,
-            (const char *)&flow->lower_addr4->sin_addr, sizeof(struct in_addr));
-        sha1_write(&ctx,
-            (const char *)&flow->upper_addr4->sin_addr, sizeof(struct in_addr));
-        break;
-    case AF_INET6:
-        sha1_write(&ctx,
-            (const char *)&flow->lower_addr6->sin6_addr, sizeof(struct in6_addr));
-        sha1_write(&ctx,
-            (const char *)&flow->upper_addr6->sin6_addr, sizeof(struct in6_addr));
-        break;
-    }
+    sha1_write(&ctx, flow->lower_addr.GetAddress(),
+        flow->lower_addr.GetAddressSize());
+    sha1_write(&ctx, flow->upper_addr.GetAddress(),
+        flow->upper_addr.GetAddressSize());
 
-    sha1_write(&ctx,
-        (const char *)&flow->lower_port, sizeof(uint16_t));
-    sha1_write(&ctx,
-        (const char *)&flow->upper_port, sizeof(uint16_t));
+    uint16_t port = flow->lower_addr.GetPort(false);
+    sha1_write(&ctx, (const char *)&port, sizeof(uint16_t));
+    port = flow->upper_addr.GetPort(false);
+    sha1_write(&ctx, (const char *)&port, sizeof(uint16_t));
 
     digest.assign((const char *)sha1_result(&ctx, _digest), SHA1_DIGEST_LENGTH);
 
