@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,15 +25,17 @@
 #include <map>
 #include <queue>
 #include <list>
+#include <set>
 #include <sstream>
 #include <stdexcept>
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
 #include <vector>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
+#include <mutex>
+#include <bitset>
 
 #include <arpa/inet.h>
 #include <errno.h>
@@ -41,16 +43,13 @@
 #include <unistd.h>
 #include <netdb.h>
 #include <net/if.h>
-#include <netinet/in.h>
-#include <netinet/ip.h>
-#include <netinet/tcp.h>
-#include <netinet/udp.h>
 #include <pthread.h>
 #include <signal.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
 #include <sys/ioctl.h>
 #include <sys/select.h>
 #include <sys/socket.h>
@@ -60,10 +59,22 @@
 #include <sys/un.h>
 #include <sys/param.h>
 
+#define __FAVOR_BSD 1
+#include <netinet/in.h>
+#include <netinet/ip.h>
+#include <netinet/ip6.h>
+#include <netinet/tcp.h>
+#include <netinet/udp.h>
+#undef __FAVOR_BSD
+
 #ifndef UNIX_PATH_MAX
 #define UNIX_PATH_MAX 104
 #endif
 
+#include <net/if.h>
+#include <net/if_arp.h>
+#include <linux/if_packet.h>
+
 #include <pcap/pcap.h>
 
 #ifdef _ND_USE_CONNTRACK
@@ -73,32 +84,42 @@
 #include <nlohmann/json.hpp>
 using json = nlohmann::json;
 
+#include <radix/radix_tree.hpp>
+
 using namespace std;
 
 #include "netifyd.h"
 
+#include "nd-config.h"
 #include "nd-ndpi.h"
+#include "nd-risks.h"
+#include "nd-serializer.h"
+#include "nd-packet.h"
+#include "nd-json.h"
+#include "nd-util.h"
+#include "nd-addr.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
-#include "nd-json.h"
+#include "nd-apps.h"
+#include "nd-protos.h"
+#include "nd-category.h"
 #include "nd-flow.h"
 #include "nd-thread.h"
 #ifdef _ND_USE_CONNTRACK
 #include "nd-conntrack.h"
 #endif
-#include "nd-util.h"
 #include "nd-signal.h"
 #include "nd-socket.h"
 
 #define _ND_SOCKET_PROC_NET_UNIX    "/proc/net/unix"
 
-extern nd_global_config nd_config;
+extern ndGlobalConfig nd_config;
 
 ndSocketLocal::ndSocketLocal(ndSocket *base, const string &node)
     : base(base), valid(false)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
     struct sockaddr_un *sa_un = new struct sockaddr_un;
 
     base->node = node;
@@ -122,7 +143,7 @@ ndSocketLocal::ndSocketLocal(ndSocket *base, const string &node)
 
 ndSocketLocal::~ndSocketLocal()
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
     if (valid && base->type == ndSOCKET_TYPE_SERVER)
         unlink(base->node.c_str());
 }
@@ -142,7 +163,7 @@ int ndSocketLocal::IsValid(void)
 #else
 int ndSocketLocal::IsValid(void)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
     struct stat socket_stat;
 
@@ -195,7 +216,7 @@ ndSocketRemote::ndSocketRemote(
     ndSocket *base, const string &node, const string &service)
     : base(base)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
     base->node = node;
     base->service = service;
@@ -205,13 +226,13 @@ ndSocketRemote::ndSocketRemote(
 
 ndSocketRemote::~ndSocketRemote()
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketClient::ndSocketClient(ndSocket *base)
     : base(base)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
     base->type = ndSOCKET_TYPE_CLIENT;
 }
@@ -222,7 +243,7 @@ ndSocketClient::~ndSocketClient()
 
 ndSocket *ndSocketServer::Accept(void)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
     ndSocket *peer = NULL;
     int peer_sd = -1;
     socklen_t peer_sa_size = 0;
@@ -250,7 +271,7 @@ ndSocket *ndSocketServer::Accept(void)
             if (peer == NULL)
                 throw ndSocketSystemException(__PRETTY_FUNCTION__, "new", ENOMEM);
 
-            nd_debug_printf("%s: peer: %s\n", __PRETTY_FUNCTION__, base->node.c_str());
+            nd_dprintf("%s: peer: %s\n", __PRETTY_FUNCTION__, base->node.c_str());
         }
         else {
             char node[NI_MAXHOST], service[NI_MAXSERV];
@@ -267,7 +288,7 @@ ndSocket *ndSocketServer::Accept(void)
             if (peer == NULL)
                 throw ndSocketSystemException(__PRETTY_FUNCTION__, "new", ENOMEM);
 
-            nd_debug_printf("%s: peer: %s:%s\n", __PRETTY_FUNCTION__, node, service);
+            nd_dprintf("%s: peer: %s:%s\n", __PRETTY_FUNCTION__, node, service);
         }
 
         peer->sd = peer_sd;
@@ -294,58 +315,58 @@ ndSocket *ndSocketServer::Accept(void)
 ndSocketServer::ndSocketServer(ndSocket *base)
     : base(base)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
     base->type = ndSOCKET_TYPE_SERVER;
 }
 
 ndSocketServer::~ndSocketServer()
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketClientLocal::ndSocketClientLocal(const string &node)
     : ndSocketClient(this), ndSocketLocal(this, node)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
 }
 
 ndSocketClientLocal::~ndSocketClientLocal()
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketServerLocal::ndSocketServerLocal(const string &node)
     : ndSocketServer(this), ndSocketLocal(this, node)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketServerLocal::~ndSocketServerLocal()
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketClientRemote::ndSocketClientRemote(const string &node, const string &service)
     : ndSocketClient(this), ndSocketRemote(this, node, service)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketClientRemote::~ndSocketClientRemote()
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketServerRemote::ndSocketServerRemote(const string &node, const string &service)
     : ndSocketServer(this), ndSocketRemote(this, node, service)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketServerRemote::~ndSocketServerRemote()
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocket::ndSocket()
@@ -353,7 +374,7 @@ ndSocket::ndSocket()
     type(ndSOCKET_TYPE_NULL), state(ndSOCKET_STATE_INIT),
     bytes_in(0), bytes_out(0)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocket::ndSocket(const string &node)
@@ -361,7 +382,7 @@ ndSocket::ndSocket(const string &node)
     node(node), type(ndSOCKET_TYPE_NULL), state(ndSOCKET_STATE_INIT),
     bytes_in(0), bytes_out(0)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocket::ndSocket(const string &host, const string &service)
@@ -370,12 +391,12 @@ ndSocket::ndSocket(const string &host, const string &service)
     type(ndSOCKET_TYPE_NULL), state(ndSOCKET_STATE_INIT),
     bytes_in(0), bytes_out(0)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 }
 
 ndSocket::~ndSocket()
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
     if (sd != -1) close(sd);
     if (sa != NULL) delete sa;
 }
@@ -385,7 +406,7 @@ ssize_t ndSocket::Read(uint8_t *buffer, ssize_t length)
     uint8_t *p = buffer;
     ssize_t bytes_read = 0, bytes_remaining = length;
 
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
     do {
         ssize_t rc = read(sd, p, bytes_remaining);
@@ -413,7 +434,7 @@ ssize_t ndSocket::Write(const uint8_t *buffer, ssize_t length)
     const uint8_t *p = buffer;
     ssize_t bytes_wrote = 0, bytes_remaining = length;
 
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
     do {
         ssize_t rc = write(sd, p, bytes_remaining);
@@ -440,7 +461,7 @@ void ndSocket::SetBlockingMode(bool enable)
 {
     int flags;
 
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
     if (enable == false) {
         flags = fcntl(sd, F_GETFL);
@@ -461,7 +482,7 @@ void ndSocket::SetBlockingMode(bool enable)
 
 void ndSocket::Create(void)
 {
-    //nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    //nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
     if (family == AF_UNSPEC) {
         struct addrinfo hints;
@@ -574,7 +595,7 @@ void ndSocket::Create(void)
         }
     }
 
-    nd_debug_printf("%s: created\n", __PRETTY_FUNCTION__);
+    nd_dprintf("%s: created\n", __PRETTY_FUNCTION__);
 }
 
 ndSocketBuffer::ndSocketBuffer()
@@ -608,7 +629,7 @@ size_t ndSocketBuffer::BufferQueueFlush(void)
 
         if (bytes_wrote < 0) {
             if (errno == EAGAIN || errno == EWOULDBLOCK) {
-                //nd_debug_printf("Unable to flush buffer queue to client socket: %s\n",
+                //nd_dprintf("Unable to flush buffer queue to client socket: %s\n",
                 //    strerror(errno));
                 break;
             }
@@ -673,8 +694,8 @@ void ndSocketBuffer::Pop(size_t bytes)
         throw ndSocketSystemException(__PRETTY_FUNCTION__, "recv(short)", EINVAL);
 }
 
-ndSocketThread::ndSocketThread()
-    : ndThread("nd-socket", -1)
+ndSocketThread::ndSocketThread(int16_t cpu)
+    : ndThread("nd-socket", (long)cpu)
 {
     vector<pair<string, string> >::const_iterator i;
     for (i = nd_config.socket_host.begin();
@@ -750,18 +771,25 @@ void ndSocketThread::ClientAccept(ndSocketServerMap::iterator &si)
     clients[client->GetDescriptor()] = client;
 
     try {
-        string json;
-        nd_json_agent_hello(json);
-        buffer->Push(json);
-        nd_json_agent_status(json);
-        buffer->Push(json);
-        nd_json_protocols(json);
-        buffer->Push(json);
+        json js;
+        string json_string;
+
+        nd_json_agent_hello(json_string);
+        buffer->Push(json_string);
+
+        js["type"] = "agent_status";
+        nd_json_agent_status(js);
+        nd_json_to_string(js, json_string);
+        json_string.append("\n");
+        buffer->Push(json_string);
+
+        nd_json_protocols(json_string);
+        buffer->Push(json_string);
     }
     catch (exception &e) {
         Unlock();
 
-        nd_debug_printf("%s: Exception while sending JSON: %s\n",
+        nd_dprintf("%s: Exception while sending JSON: %s\n",
             tag.c_str(), e.what());
 
         delete client;
@@ -778,7 +806,7 @@ void ndSocketThread::ClientHangup(ndSocketClientMap::iterator &ci)
 {
     ndSocketBufferMap::iterator bi;
 
-    nd_debug_printf("%s\n", __PRETTY_FUNCTION__);
+    nd_dprintf("%s\n", __PRETTY_FUNCTION__);
 
     delete ci->second;
     bi = buffers.find(ci->first);
@@ -798,6 +826,17 @@ void ndSocketThread::ClientHangup(ndSocketClientMap::iterator &ci)
     }
 }
 
+size_t ndSocketThread::GetClientCount(void)
+{
+    size_t client_count = 0;
+
+    Lock();
+    client_count = clients.size();
+    Unlock();
+
+    return client_count;
+}
+
 void *ndSocketThread::Entry(void)
 {
     struct timeval tv;
@@ -806,9 +845,9 @@ void *ndSocketThread::Entry(void)
     ndSocketServerMap::iterator si;
     ndSocketBufferMap::iterator bi;
 
-    nd_debug_printf("%s: started\n", __PRETTY_FUNCTION__);
+    nd_dprintf("%s: started\n", __PRETTY_FUNCTION__);
 
-    while (! terminate) {
+    while (! ShouldTerminate()) {
         int rc_read = -1, rc_write = -1;
         int max_read_fd = -1, max_write_fd = -1;
 
